#include <mega16.h>
#include <delay.h>

// I2C Bus functions
#asm
   .equ __i2c_port=0x15 ;PORTC
   .equ __sda_bit=1
   .equ __scl_bit=0
#endasm
#include <i2c.h>

// PCF8583 Real Time Clock functions
#include "PCF8583.h"

// Alphanumeric LCD Module functions
#include <alcd.h>

#ifndef RXB8
#define RXB8 1
#endif

#ifndef TXB8
#define TXB8 0
#endif

#ifndef UPE
#define UPE 2
#endif

#ifndef DOR
#define DOR 3
#endif

#ifndef FE
#define FE 4
#endif

#ifndef UDRE
#define UDRE 5
#endif

#ifndef RXC
#define RXC 7
#endif

#define FRAMING_ERROR (1<<FE)
#define PARITY_ERROR (1<<UPE)
#define DATA_OVERRUN (1<<DOR)
#define DATA_REGISTER_EMPTY (1<<UDRE)
#define RX_COMPLETE (1<<RXC)

// USART Receiver buffer
#define RX_BUFFER_SIZE 8
char rx_buffer[RX_BUFFER_SIZE];

#if RX_BUFFER_SIZE <= 256
unsigned char rx_wr_index,rx_rd_index,rx_counter;
#else
unsigned int rx_wr_index,rx_rd_index,rx_counter;
#endif

// This flag is set on USART Receiver buffer overflow
bit rx_buffer_overflow;

// USART Receiver interrupt service routine
interrupt [USART_RXC] void usart_rx_isr(void)
{
char status,data;
status=UCSRA;
data=UDR;
if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
   {
   rx_buffer[rx_wr_index++]=data;
#if RX_BUFFER_SIZE == 256
   // special case for receiver buffer size=256
   if (++rx_counter == 0)
      {
#else
   if (rx_wr_index == RX_BUFFER_SIZE) rx_wr_index=0;
   if (++rx_counter == RX_BUFFER_SIZE)
      {
      rx_counter=0;
#endif
      rx_buffer_overflow=1;
      }
   }
}

#ifndef _DEBUG_TERMINAL_IO_
// Get a character from the USART Receiver buffer
#define _ALTERNATE_GETCHAR_
#pragma used+
char getchar(void)
{
char data;
while (rx_counter==0);
data=rx_buffer[rx_rd_index++];
#if RX_BUFFER_SIZE != 256
if (rx_rd_index == RX_BUFFER_SIZE) rx_rd_index=0;
#endif
#asm("cli")
--rx_counter;
#asm("sei")
return data;
}
#pragma used-
#endif

// USART Transmitter buffer
#define TX_BUFFER_SIZE 8
char tx_buffer[TX_BUFFER_SIZE];

#if TX_BUFFER_SIZE <= 256
unsigned char tx_wr_index,tx_rd_index,tx_counter;
#else
unsigned int tx_wr_index,tx_rd_index,tx_counter;
#endif

// USART Transmitter interrupt service routine
interrupt [USART_TXC] void usart_tx_isr(void)
{
if (tx_counter)
   {
   --tx_counter;
   UDR=tx_buffer[tx_rd_index++];
#if TX_BUFFER_SIZE != 256
   if (tx_rd_index == TX_BUFFER_SIZE) tx_rd_index=0;
#endif
   }
}

#ifndef _DEBUG_TERMINAL_IO_
// Write a character to the USART Transmitter buffer
#define _ALTERNATE_PUTCHAR_
#pragma used+
void putchar(char c)
{
while (tx_counter == TX_BUFFER_SIZE);
#asm("cli")
if (tx_counter || ((UCSRA & DATA_REGISTER_EMPTY)==0))
   {
   tx_buffer[tx_wr_index++]=c;
#if TX_BUFFER_SIZE != 256
   if (tx_wr_index == TX_BUFFER_SIZE) tx_wr_index=0;
#endif
   ++tx_counter;
   }
else
   UDR=c;
#asm("sei")
}
#pragma used-
#endif

// Standard Input/Output functions
#include <stdio.h>

#define RELAY1 PORTD.7
#define RELAY2 PORTD.6
#define MAXPARAMETERS 50
#define LIGHTON 1
#define LIGHTOFF 2

#define ON 1
#define OFF 0

#define Cancel_BTN PINB.0
#define Plus_BTN PINB.1
#define Minus_BTN PINB.2
#define Agree_BTN PINB.3

unsigned int mSec, clock_mSec, Ring_mSec;
char State, Button, Counter, i, LightFlag;
char Hour, Minute, Seconds, Ring_Minute, mSeconds;

eeprom unsigned char Parameters[MAXPARAMETERS][2] = {0};

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
  TCNT0=0x83;
  mSec++;
  clock_mSec++;
}

// Timer2 overflow interrupt service routine
interrupt [TIM2_OVF] void timer2_ovf_isr(void)
{
  TCNT2=0x83;

}

void lcd_putvalue(char Value, char X, char Y)
{
  lcd_gotoxy(X, Y);
  lcd_putchar(Value / 10 + 48);
  Value %= 10;
  lcd_putchar(Value + 48);
  lcd_gotoxy(0, 0);
}

void lcd_putParameterIndex(char Value, char X, char Y)
{
  lcd_gotoxy(X, Y);
  lcd_putchar('P');
  lcd_putchar(Value / 10 + 48);
  Value %= 10;
  lcd_putchar(Value + 48);
  lcd_gotoxy(0, 0);
}

void lcd_putstring(unsigned char* string)
{
  while (*string) lcd_putchar(*string++);
}

void MemGuard(void)
{
  char i;
  for (i = 0; i < MAXPARAMETERS; i++)
  {
    if (Parameters[i][0] > 0xFE) Parameters[i][0] = 0;
    if (Parameters[i][1] > 0xFE) Parameters[i][1] = 0;
  }
}

char PressedButton(void)
{
  char result = 0;
  if (Agree_BTN == 0) result = 'A';
  if (Plus_BTN == 0) result = '+';
  if (Minus_BTN == 0) result = '-';
  if (Cancel_BTN == 0) result = 'C';
  return result;
}

void Light(char Value)
{
  RELAY2 = Value;
}

void RingBell(char Value)
{
  RELAY1 = Value;
}

void main(void)
{
{
// Input/Output Ports initialization
// Port A initialization
// Func7=Out Func6=Out Func5=Out Func4=Out Func3=Out Func2=Out Func1=Out Func0=Out
// State7=0 State6=0 State5=0 State4=0 State3=0 State2=0 State1=0 State0=0
PORTA=0x00;
DDRA=0xFF;

// Port B initialization
// Func7=Out Func6=Out Func5=Out Func4=Out Func3=In Func2=In Func1=In Func0=In
// State7=0 State6=0 State5=0 State4=0 State3=T State2=T State1=T State0=T
PORTB=0b00001111;
DDRB=0xF0;

// Port C initialization
// Func7=Out Func6=Out Func5=Out Func4=Out Func3=Out Func2=Out Func1=In Func0=In
// State7=0 State6=0 State5=0 State4=0 State3=0 State2=0 State1=T State0=T
PORTC=0x00;
DDRC=0xFC;

// Port D initialization
// Func7=Out Func6=Out Func5=Out Func4=Out Func3=In Func2=Out Func1=Out Func0=In
// State7=0 State6=0 State5=0 State4=0 State3=T State2=0 State1=0 State0=T
PORTD=0x00;
DDRD=0xF6;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 125,000 kHz
// Mode: Normal top=0xFF
// OC0 output: Disconnected
TCCR0=0x03;
TCNT0=0x83;
OCR0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer1 Stopped
// Mode: Normal top=0xFFFF
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: 125,000 kHz
// Mode: Normal top=0xFF
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x04;
TCNT2=0x83;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
// INT2: Off
MCUCR=0x00;
MCUCSR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x01;

// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: On
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud Rate: 9600
UCSRA=0x00;
UCSRB=0xD8;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x33;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;

// ADC initialization
// ADC disabled
ADCSRA=0x00;

// SPI initialization
// SPI disabled
SPCR=0x00;

// TWI initialization
// TWI disabled
TWCR=0x00;

// I2C Bus initialization
i2c_init();

// PCF8583 Real Time Clock initialization
rtc_init(1,0);

// Alphanumeric LCD initialization
// Connections specified in the
// Project|Configure|C Compiler|Libraries|Alphanumeric LCD menu:
// RS - PORTA Bit 0
// RD - PORTA Bit 1
// EN - PORTA Bit 2
// D4 - PORTA Bit 4
// D5 - PORTA Bit 5
// D6 - PORTA Bit 6
// D7 - PORTA Bit 7
// Characters/line: 16
lcd_init(16);

// Global enable interrupts
#asm("sei")
}
MemGuard();
lcd_clear();
while (1)
      {
        Button = PressedButton();
        if (Button > 0) delay_ms(200);
        {
        switch(State)
        {
          case 0 : //   
            lcd_putParameterIndex(Counter, 1, 0);
            switch(Button)
            {
              case '-' :
                if (Counter > 0) Counter--;
                if (Counter == 0) Counter = MAXPARAMETERS;
              break;
              case '+' :
                if (Counter < 1 + MAXPARAMETERS) Counter++;
                if (Counter == 1 + MAXPARAMETERS) Counter = 0;
              break;
              case 'A' : State = 1;
              break;
              case 'C' :
              break;
            }
          break;
          case 1 : //    
            if (Counter == 0) //  
            {
              lcd_putvalue(Hour, 5, 0);
              switch(Button)
              {
                case '-' : if (Hour > 0) Hour--; if (Hour == 0) Hour = 23;
                break;
                case '+' : if (Hour < 23) Hour++; if (Hour == 23) Hour = 0;
                break;
                case 'A' : State = 2; rtc_set_time(1, Hour, Minute, 0, 0);
                break;
                case 'C' : State = 0; lcd_clear();
                break;
              }
            }
            else
            {
              lcd_putvalue(Parameters[Counter-1][0], 5, 0);
              switch(Button)
              {
                case '-' :
                  if (Parameters[Counter-1][0] > 0) Parameters[Counter-1][0]--;
                  if (Parameters[Counter-1][0] == 0) Parameters[Counter-1][0] = 23;
                break;
                case '+' :
                  if (Parameters[Counter-1][0] < 23) Parameters[Counter-1][0]++;
                  if (Parameters[Counter-1][0] == 23) Parameters[Counter-1][0] = 0;
                break;
                case 'A' : State = 2;
                break;
                case 'C' : State = 0; lcd_clear();
                break;
              }
            }
          break;
          case 2 : //    
            if (Counter == 0) //  
            {
              lcd_putvalue(Minute, 8, 0);
              switch(Button)
              {
                case '-' : if (Minute > 0) Minute--; if (Minute == 0) Minute = 59;
                break;
                case '+' : if (Minute < 59) Minute++; if (Minute == 59) Minute = 0;
                break;
                case 'A' : State = 0; rtc_set_time(1, Hour, Minute, 0, 0); lcd_clear();
                break;
                case 'C' : State = 1;
                break;
              }
            }
            else
            {
              lcd_putvalue(Parameters[Counter-1][1], 8, 0);
              switch(Button)
              {
                case '-' :
                  if (Parameters[Counter-1][1] > 0) Parameters[Counter-1][1]--;
                  if (Parameters[Counter-1][1] == 0) Parameters[Counter-1][1] = 59;
                break;
                case '+' :
                  if (Parameters[Counter-1][1] < 59) Parameters[Counter-1][1]++;
                  if (Parameters[Counter-1][1] == 59) Parameters[Counter-1][1] = 0;
                break;
                case 'A' : State = 0; lcd_clear();
                break;
                case 'C' : State = 1;
                break;
              }
            }
          break;
        }
        }
          if (mSec > 499)
          {
            lcd_gotoxy(13, 1);
            lcd_putchar(' ');
          }
          if (mSec > 999)
          {
            if (State == 0) rtc_get_time(1, &Hour, &Minute, &Seconds, &mSeconds);
            lcd_putvalue(Hour, 11, 1);
            lcd_putvalue(Minute, 14, 1);
            mSec = 0;
            lcd_gotoxy(13, 1);
            lcd_putchar(':');
          }

        if (clock_mSec > 4999)
        {
          clock_mSec = 0;
          if (State == 0) rtc_get_time(1, &Hour, &Minute, &Seconds, &mSeconds);
          if (Hour >= Parameters[LIGHTON][0])
          {
            if (Hour < Parameters[LIGHTOFF][0])
            {
              if (Minute >= Parameters[LIGHTON][1])
              {
                Light(ON);
                lcd_putvalue(Parameters[LIGHTON][0], 0, 1);
                lcd_gotoxy(2, 1);
                lcd_putchar(':');
                lcd_putvalue(Parameters[LIGHTON][1], 3, 1);
              }
            }
            if (Hour >= Parameters[LIGHTOFF][0])
            {
              if (Minute >= Parameters[LIGHTOFF][1])
              {
                Light(OFF);
                lcd_putvalue(Parameters[LIGHTOFF][0], 0, 1);
                lcd_gotoxy(2, 1);
                lcd_putchar(':');
                lcd_putvalue(Parameters[LIGHTOFF][1], 3, 1);
              }
              else
              {
                Light(ON);
                lcd_putvalue(Parameters[LIGHTON][0], 0, 1);
                lcd_gotoxy(2, 1);
                lcd_putchar(':');
                lcd_putvalue(Parameters[LIGHTON][1], 3, 1);
              }
            }
          }
          for(i = 3; i < MAXPARAMETERS; i++)
          {
            if (Parameters[i][0] == Hour) if (Parameters[i][1] == Minute) if (Seconds - 5 < Parameters[0][1]) RingBell(ON);
            else RingBell(OFF);
          }
        }

      }
}
